package com.gic.haoban.manage.service.task;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.clerk.api.dto.ClerkDTO;
import com.gic.clerk.api.dto.ClerkListDTO;
import com.gic.clerk.api.service.ClerkService;
import com.gic.commons.util.GICMQClientUtil;
import com.gic.enterprise.api.dto.StoreDTO;
import com.gic.enterprise.api.service.StoreGroupService;
import com.gic.enterprise.api.service.StoreService;
import com.gic.haoban.base.api.common.Constant;
import com.gic.haoban.base.api.common.ServiceResponse;
import com.gic.haoban.manage.api.dto.*;
import com.gic.haoban.manage.api.enums.PreDealStatusEnum;
import com.gic.haoban.manage.api.enums.PreDealTypeEnum;
import com.gic.haoban.manage.api.enums.SyncTaskStatusEnum;
import com.gic.haoban.manage.api.service.DealSyncOperationApiService;
import com.gic.haoban.manage.api.service.DepartmentApiService;
import com.gic.haoban.manage.api.service.StaffApiService;
import com.gic.haoban.manage.service.entity.TabHaobanDepartment;
import com.gic.haoban.manage.service.entity.TabHaobanPreDealLog;
import com.gic.haoban.manage.service.entity.TabHaobanSyncTask;
import com.gic.haoban.manage.service.service.DepartmentService;
import com.gic.haoban.manage.service.service.PreDealService;
import com.gic.haoban.manage.service.service.StaffService;
import com.gic.haoban.manage.service.service.SyncTaskService;
import com.gic.mq.sdk.GicMQClient;
import com.gic.redis.data.util.RedisUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * Created 2020/4/13.
 *
 * @author hua
 */
@Service
public class ClerkSyncOperation implements BaseSyncOperation {
    private static final Logger logger = LoggerFactory.getLogger(ClerkSyncOperation.class);

    @Autowired
    private DepartmentService departmentService;

    @Autowired
    private StoreGroupService storeGroupService;

    @Autowired
    private PreDealService preDealService;

    @Autowired
    private DepartmentApiService departmentApiService;

    @Autowired
    private SyncTaskService syncTaskService;

    @Autowired
    private StoreService storeService;

    @Autowired
    private ClerkService clerkService;

    @Autowired
    private StaffApiService staffApiService;

    @Autowired
    private DealSyncOperationApiService dealSyncOperationApiService;

    @Override
    public void dealSingleByMq(DealParamMqDTO dealParamMqDTO, TabHaobanPreDealLog dataPre) {
        logger.info("成员处理：{}", JSONObject.toJSONString(dealParamMqDTO));
        ClerkDTO clerkDTO = clerkService.getClerkByClerkIdNodel(dealParamMqDTO.getData());
        String reason = "";
        boolean dealFlag = true;
        if (clerkDTO == null) {
            logger.info("成员不存在:t:{},d:{}", dealParamMqDTO.getTaskId(), dealParamMqDTO.getData());
            dealException(dealParamMqDTO.getTaskId(), dealParamMqDTO.getData(), dataPre.getEnterpriseId(), "成员不存在");
            return;
        }
        //处理clerk
        try {
            GicClerkDTO gicClerkDTO = new GicClerkDTO();
            gicClerkDTO.setNationcode(clerkDTO.getNationcode());
            gicClerkDTO.setClerkCode(clerkDTO.getClerkCode());
            gicClerkDTO.setPostionName(clerkDTO.getPositionName());
            gicClerkDTO.setHeadImgUrl(clerkDTO.getHeadImgUrl());
            gicClerkDTO.setPhoneNumber(clerkDTO.getPhoneNumber());
            gicClerkDTO.setStatus(clerkDTO.getStatus());
            if (!StringUtils.isBlank(clerkDTO.getClerkGender())) {
                gicClerkDTO.setClerkGender(Integer.valueOf(clerkDTO.getClerkGender()));
            }
            gicClerkDTO.setClerkName(clerkDTO.getClerkName());
            gicClerkDTO.setStoreId(clerkDTO.getStoreId());
            gicClerkDTO.setEnterpriseId(clerkDTO.getEnterpriseId());
            ServiceResponse response = staffApiService.saveSyncStaffByGic(gicClerkDTO);
            if (response.getCode() != 1) {
                reason = response.getMessage();
                dealFlag = false;
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.info("同步失败：", JSONObject.toJSONString(dataPre));
            reason = "成员处理异常";
            dealFlag = false;
        } finally {
            if (!dealFlag) {
                dealException(dealParamMqDTO.getTaskId(), dealParamMqDTO.getData(), dataPre.getEnterpriseId(), reason);
            } else {
                dealSuccess(dealParamMqDTO.getTaskId(), dealParamMqDTO.getData(), dataPre.getEnterpriseId(), dataPre.getWxEnterpriseId());
            }
        }


    }

    @Override
    public void dealException(String taskId, String dataId, String enterpriseId, String reason) {
        logger.info("成员处理失败：t:{},d:{}", taskId, dataId);
        boolean b = preDealService.updateStatusByDataId(taskId, dataId, PreDealStatusEnum.exception.getVal(), reason);
        if (!b) {
            logger.info("成员处理异常:t:{}, d:{}", taskId, dataId);
            return;
        }
        checkDepartmentTask(taskId);

    }

    @Override
    public void dealSuccess(String taskId, String dataId, String enterpriseId, String wxEnterpriseId) {
        logger.info("成员处理成功：t:{},d:{}", taskId, dataId);
        boolean b = preDealService.updateStatusByDataId(taskId, dataId, PreDealStatusEnum.computed.getVal(), "成功");
        if (!b) {
            logger.info("成功修改错误！{}", dataId);
            return;
        }
        checkDepartmentTask(taskId);
    }

    @Override
    public void checkDepartmentTask(String taskId) {
        boolean b = preDealService.checkTask(taskId, PreDealTypeEnum.clerk.getVal());
        if (!b) {
            return;
        }
        logger.info("店员同步成功:{}", taskId);
        String key = "haoban_sync_clerk_task_" + taskId;
        RedisUtil.lock(key, 3L);
        TabHaobanSyncTask syncTask = syncTaskService.getSyncTask(taskId);
        //同步成功 进入门店处理
        if (syncTask.getStatusFlag().equals(SyncTaskStatusEnum.clerk_sync.getVal())) {
            int errCount = preDealService.countByTaskId(taskId, -1, PreDealStatusEnum.exception.getVal());
            if (errCount > 0) {
                syncTaskService.updateTaskStatus(taskId, SyncTaskStatusEnum.exception_compute.getVal());
            } else {
                syncTaskService.updateTaskStatus(taskId, SyncTaskStatusEnum.compute.getVal());
                dealSyncOperationApiService.unlockTask(syncTask.getWxEnterpriseId());
            }
        }
        RedisUtil.unlock(key);
    }


}
