package com.gic.haoban.manage.web.controller;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import cn.hutool.core.collection.CollectionUtil;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.ctrip.framework.apollo.spring.annotation.ApolloConfig;
import com.gic.api.base.commons.Page;
import com.gic.enterprise.api.dto.StoreDTO;
import com.gic.enterprise.api.service.StoreService;
import com.gic.haoban.base.api.common.BasePageInfo;
import com.gic.haoban.common.utils.GlobalVar;
import com.gic.haoban.manage.api.dto.*;
import com.gic.haoban.manage.api.enums.StoreFieldEnum;
import com.gic.haoban.manage.api.service.ApplicationSettingApiService;
import com.gic.haoban.manage.api.service.AuditApiService;
import com.gic.haoban.manage.api.service.AuditSettingApiService;
import com.gic.haoban.manage.api.service.DepartmentApiService;
import com.gic.haoban.manage.api.service.StaffApiService;
import com.gic.haoban.manage.api.service.WxApplicationApiService;
import com.gic.haoban.manage.api.service.WxEnterpriseApiService;
import com.gic.haoban.manage.web.anno.HttpLimit;
import com.gic.haoban.manage.web.anno.IgnoreLogin;
import com.gic.haoban.manage.web.auth.AuthRequestUtil;
import com.gic.haoban.manage.web.config.Config;
import com.gic.haoban.manage.web.vo.LoginVO;
import com.gic.haoban.manage.web.vo.StaffVO;
import com.gic.wechat.api.service.qywx.QywxSuiteApiService;
import com.gic.wechat.api.service.qywx.QywxUserApiService;

import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import com.gic.haoban.common.utils.HaobanResponse;
import com.gic.haoban.manage.api.service.ApplicationApiService;
import com.gic.haoban.manage.web.errCode.HaoBanErrCode;

@RestController
@RequestMapping("/audit")
public class AuditController extends WebBaseController{
	private static final Logger log = LogManager.getLogger(AuditController.class);
	
	@Autowired
	private AuditSettingApiService auditSettingApiService;
	@Autowired
	private AuditApiService auditApiService;
	@Autowired
	private StaffApiService staffApiService;
	@Autowired
	private StoreService storeService;
	
	//设置保存
	@HttpLimit
	@RequestMapping("open-or-close")
	public HaobanResponse openOrClose(String wxEnterpriseId,Integer auditFlag) {
		if(StringUtils.isAnyBlank(wxEnterpriseId)){
			return resultResponse(HaoBanErrCode.ERR_2);
		}
		if(auditFlag == null){
			return resultResponse(HaoBanErrCode.ERR_2);
		}
		auditSettingApiService.saveSetting(wxEnterpriseId, auditFlag);
		return resultResponse(HaoBanErrCode.ERR_1);
	}
	//设置查看
	@RequestMapping("find-setting")
	public HaobanResponse findSetting(String wxEnterpriseId ) {
		if(StringUtils.isAnyBlank(wxEnterpriseId)){
			return resultResponse(HaoBanErrCode.ERR_2);
		}
		AuditSettingDTO auditSettingDTO = auditSettingApiService.findSettingByWxEnterpriseId(wxEnterpriseId);
		if(auditSettingDTO == null){
			auditSettingDTO = new AuditSettingDTO();
			auditSettingDTO.setAuditFlag(1);
		}
		return resultResponse(HaoBanErrCode.ERR_1,auditSettingDTO);
	}
	//分页查找
	@RequestMapping("find-page")
	public HaobanResponse findPage(String wxEnterpriseId,String enterpriseId,String search,BasePageInfo pageInfo ,Integer auditType,Integer auditStatus,Integer auditFlag ) {
		if(auditFlag == null){
			return resultResponse(HaoBanErrCode.ERR_2);
		}
		if(StringUtils.isAnyBlank(wxEnterpriseId)){
			return resultResponse(HaoBanErrCode.ERR_2);
		}
		Page<AuditDTO> page = auditApiService.page(auditType, search, wxEnterpriseId,enterpriseId, auditStatus,auditFlag, pageInfo);
		List<AuditDTO> list = page.getResult();
		List<String> storeIds = list.stream().map(s->s.getCommitStoreId()).collect(Collectors.toList());
		String[] array=storeIds.toArray(new String[storeIds.size()]);
		List<StoreDTO> stores = storeService.getStores(array);
		Map<String,StoreDTO> map = com.gic.commons.util.CollectionUtil.toMap(stores, "storeId");
		for(AuditDTO dto:list){
			String t = dto.getChangeField();
			String detail = "";
			if(StoreFieldEnum.STORE_NAME.getValue().equals(t)){
				detail = "将"+"门店名称"+"\""+dto.getOldValue()+"\""+"修改为"+"\""+dto.getNewValue()+"\"";
			}else if(StoreFieldEnum.STORE_AREA.getValue().equals(t)){
				detail = "将"+"门店面积"+"\""+dto.getOldValue()+"\""+"修改为"+"\""+dto.getNewValue()+"\"";
			}else if(StoreFieldEnum.CONACTS_PHONE.getValue().equals(t)){
				detail = "将"+"联系电话"+"\""+dto.getOldValue()+"\""+"修改为"+"\""+dto.getNewValue()+"\"";
			}else if(StoreFieldEnum.BUSINESS_TIME_STRING.getValue().equals(t)){
				detail = "将"+"营业时间"+"\""+dto.getOldValue()+"\""+"修改为"+"\""+dto.getNewValue()+"\"";
			}else if(StoreFieldEnum.STORE_ADDRESS.getValue().equals(t)){
				String oldValue = dto.getOldValue();
				String newValue = dto.getNewValue();
				JSONObject oldJSONObject = JSONObject.parseObject(oldValue);
				String oldAddress = oldJSONObject.get("storeAddress").toString();
				
				JSONObject newJSONObject = JSONObject.parseObject(newValue);
				String newAddress = newJSONObject.get("storeAddress").toString();
				detail = "将"+"门店地址"+"\""+oldAddress+"\""+"修改为"+"\""+newAddress+"\"";
			}else if(StoreFieldEnum.STORE_IMAG.getValue().equals(t)){
				detail = "门店图片变更";
			}
			dto.setDetail(detail);
			String storeName = map.get(dto.getCommitStoreId())==null?"":map.get(dto.getCommitStoreId()).getStoreName();
			dto.setCommitStoreName(storeName);
		}
		page.setResult(list);
		return resultResponse(HaoBanErrCode.ERR_1,page);
	}
	
	//获取详情
	@RequestMapping("get-staff-detail")
	public HaobanResponse getStaffDetail(String staffId ) {
		if(StringUtils.isAnyBlank(staffId)){
			return resultResponse(HaoBanErrCode.ERR_2);
		}
		StaffDTO staffDTO = staffApiService.selectById(staffId);
//		StaffVO staffVO = new StaffVO();
//		staffVO.setPhoneNumber(phoneNumber);
//		staffVO.setNationCode(nationCode);
//		staffVO.setPostion(postion);
		return resultResponse(HaoBanErrCode.ERR_1,staffDTO);
	}
	
	
	
	//审核拒绝
	@HttpLimit
	@RequestMapping("refuse")
	public HaobanResponse refuse(String auditId,String auditReason ,String auditName) {
		if(StringUtils.isAnyBlank(auditId,auditReason,auditName)){
			return resultResponse(HaoBanErrCode.ERR_2);
		}
		auditApiService.refuse(auditId, auditReason,auditName);
		return resultResponse(HaoBanErrCode.ERR_1);
	}
	//审核同意
	@HttpLimit
	@RequestMapping("audit")
	public HaobanResponse audit(String auditId,String auditName ) {
		if(StringUtils.isAnyBlank(auditId)){
			return resultResponse(HaoBanErrCode.ERR_2);
		}
		auditApiService.audit(auditId,auditName);
		return resultResponse(HaoBanErrCode.ERR_1);
	}
}
