package com.gic.haoban.manage.web.controller;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.dubbo.common.URL;
import com.alibaba.fastjson.JSON;
import com.gic.commons.util.ToolUtil;
import com.gic.commons.web.ResponseThreadLocal;
import com.gic.haoban.common.utils.HaobanResponse;
import com.gic.haoban.common.utils.StringUtil;
import com.gic.haoban.manage.api.dto.StaffDTO;
import com.gic.haoban.manage.api.dto.WxEnterpriseDTO;
import com.gic.haoban.manage.api.service.StaffApiService;
import com.gic.haoban.manage.api.service.WxEnterpriseApiService;
import com.gic.haoban.manage.web.anno.IgnoreLogin;
import com.gic.haoban.manage.web.auth.AuthRequestUtil;
import com.gic.haoban.manage.web.config.Config;
import com.gic.haoban.manage.web.errCode.HaoBanErrCode;
import com.gic.haoban.manage.web.vo.LoginVO;
import com.gic.redis.data.util.RedisUtil;
import com.gic.wechat.api.dto.qywx.UserDTO;
import com.gic.wechat.api.service.qywx.QywxCorpApiService;
import com.gic.wechat.api.service.qywx.QywxUserApiService;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.List;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletResponse;


@RestController
public class LoginController extends WebBaseController{
	
	private static Logger logger= LoggerFactory.getLogger(LoginController.class);
	@Autowired
	StaffApiService staffApiService;
	
	@Autowired
	private QywxCorpApiService qywxCorpApiService;
	
	@Autowired
	private WxEnterpriseApiService wxEnterpriseApiService;
	
	
	@Autowired
	private Config config;
	
	@RequestMapping("login")
	public void login(String auth_code,HttpServletResponse response) {
		String userJson = qywxCorpApiService.getLoginUserInfo(config.getCorpid(), auth_code);
		if(StringUtils.isBlank(userJson)){
			return ;
		}
		UserDTO user  = JSON.parseObject(userJson, UserDTO.class);
		String corpId = user.getCorpid();
		logger.info("【企业微信登录】userId={},auth_code={},corpId={}",user.getUserid(),auth_code,corpId);
		WxEnterpriseDTO enterprise = wxEnterpriseApiService.getEnterpriseBycorpId(corpId);
		logger.info("【企业微信】enterprise={}",JSON.toJSONString(enterprise));
		StaffDTO loginStaff = null;
		if(enterprise != null){
			loginStaff = staffApiService.selectByUserIdAndEnterpriseId(user.getUserid(),enterprise.getWxEnterpriseId());
		}
		
		if(loginStaff == null){
			if(enterprise != null){
				staffApiService.wxGetAdd(user.getUserid(),enterprise.getWxEnterpriseId());
				loginStaff = staffApiService.selectByUserIdAndEnterpriseId(user.getUserid(),enterprise.getWxEnterpriseId());
			}
		}

		LoginVO loginVO = new LoginVO();
//		StaffDTO staff = staffApiService.selectById("1");
		logger.info("【好办登录】staff={}",JSON.toJSONString(loginStaff));
		if(loginStaff == null){
			return;
		}
		loginVO.setWxEnterpriseId(loginStaff.getWxEnterpriseId());
		loginVO.setStaffDTO(loginStaff);
		AuthRequestUtil.setSessionUser(loginVO);
		String redirectUri = config.getHost() + "index";
		
		redirectUri = redirectUri.replace("haoban-manage3-web", "haoban-3/#");
		String token = ToolUtil.randomUUID() + "-" + ToolUtil.randomStringByTime();
		AuthRequestUtil.setAppLoginUser(token, loginVO);
		try {
            Cookie cookie = new Cookie("token", token);
            cookie.setPath("/");
            response.addCookie(cookie);
			response.sendRedirect(redirectUri);
			response.setHeader("token", token);
		} catch (IOException e) {
			logger.warn("异常", e);
		}
		
	}
	
	@RequestMapping("login_local")
	public HaobanResponse login() {

		LoginVO loginVO = new LoginVO();
		StaffDTO staff = staffApiService.selectById("2205f92cdb5f468cb996e91dded414c2");
		String token = ToolUtil.randomUUID() + "-" + ToolUtil.randomStringByTime();
		logger.info("【好办登录】staff={}",JSON.toJSONString(staff));
		loginVO.setWxEnterpriseId(staff.getWxEnterpriseId());
		loginVO.setStaffDTO(staff);
		AuthRequestUtil.setAppLoginUser(token, loginVO);
		Cookie cookie = new Cookie("token", token);
        cookie.setPath("/");
		return resultResponse(HaoBanErrCode.ERR_1,loginVO);
	}
	
	@RequestMapping("get-login-info")
	public HaobanResponse getLoginInfo(){
		LoginVO loginVO = (LoginVO) AuthRequestUtil.getLoginUser();
		return resultResponse(HaoBanErrCode.ERR_1,loginVO);
	}
	
	@RequestMapping("get-login-qrcode")
	public HaobanResponse getLoginQrcode() throws UnsupportedEncodingException {
		String corpid = config.getCorpid();
		String redirectUri = config.getHost() + "login";
		String url = "https://open.work.weixin.qq.com/wwopen/sso/3rd_qrConnect?appid="+corpid+"&redirect_uri="+ URLEncoder.encode(redirectUri, "UTF-8")+"&state=web_login@gyoss9&usertype=admin";
		//String url = qywxCorpApiService.getGrantAuthorizationUrl(siteId, redirectUri);
		return resultResponse(HaoBanErrCode.ERR_1,URL.decode(url));
		
	}
	//运维登录
	@IgnoreLogin
	@RequestMapping("yw-login")
	public HaobanResponse ywLogin(String wxEnterpriseId,String random,HttpServletResponse response) {
		//判断是否存在random
		Object o = RedisUtil.getCache(random);
		if(o == null){
			return resultResponse(HaoBanErrCode.ERR_4);
		}
		//TODO 获取超级管理员账号接口
		StaffDTO staff = staffApiService.selectSuperByWxEnterpriseId(wxEnterpriseId);
//		StaffDTO staff = staffApiService.selectById("1");
		LoginVO loginVO = new LoginVO();
		loginVO.setWxEnterpriseId(wxEnterpriseId);
		loginVO.setStaffDTO(staff);
		AuthRequestUtil.setSessionUser(loginVO);
		String redirectUri = config.getHost() + "index";
		redirectUri = redirectUri.replace("haoban-manage3-web", "haoban-3/#");
		String token = ToolUtil.randomUUID() + "-" + ToolUtil.randomStringByTime();
		AuthRequestUtil.setAppLoginUser(token, loginVO);
		try {
			Cookie cookie = new Cookie("token", token);
            cookie.setPath("/");
            response.addCookie(cookie);
			response.sendRedirect(redirectUri);
			response.setHeader("token", token);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}	
		return resultResponse(HaoBanErrCode.ERR_1);

	}
	
	@RequestMapping("login-out")
	public HaobanResponse loginOut() {
		
		AuthRequestUtil.delSessionUser();
		return resultResponse(HaoBanErrCode.ERR_1);
		
	}

}
