package com.gic.haoban.manage.web.interceptor;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.Map;

import com.alibaba.fastjson.JSONObject;
import com.gic.haoban.common.utils.HaobanResponse;
import com.gic.haoban.manage.web.anno.HttpLimit;
import com.gic.haoban.manage.web.errCode.HaoBanErrCode;
import com.gic.redis.data.util.RedisUtil;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * 测试登录拦截器
 *
 * @author zhurz
 */
public class HttpLimitInterceptor extends HandlerInterceptorAdapter {

    private static Logger logger = LoggerFactory.getLogger(HttpLimitInterceptor.class);

    private void errorResult(HttpServletResponse httpServletResponse, HaoBanErrCode errCode) {
        HaobanResponse response = new HaobanResponse();
        response.setErrorCode(errCode.getCode());
        response.setMessage(errCode.getMsg());
        PrintWriter writer = null;
        try {
            httpServletResponse.setHeader("Content-type", "application/json;charset=UTF-8");
            writer = httpServletResponse.getWriter();
            writer.append(JSONObject.toJSONString(response));
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
    @Override
    public boolean preHandle(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse, Object o) throws Exception {
        String requestURI = httpServletRequest.getRequestURI();
        logger.info("post-url:{}", requestURI);
        Map<String, String[]> parameterMap = httpServletRequest.getParameterMap();
        logger.info("post-params:{}", JSONObject.toJSONString(parameterMap));
        if (!(o instanceof HandlerMethod)) {
            return true;
        }
        HandlerMethod handler = (HandlerMethod) o;
        HttpLimit httpLimit = handler.getMethodAnnotation(HttpLimit.class);
        if (httpLimit != null) {
            String ip = getIpAddress(httpServletRequest);
            String url = httpServletRequest.getRequestURL().toString();
            String key = "http_limit_".concat(url);
            key = key + ip;
            if(RedisUtil.getCache(key)!=null){
                this.errorResult(httpServletResponse, HaoBanErrCode.ERR_10008);
                logger.info("过于频繁点击");
            	return false;
            }else{
            	RedisUtil.setCache(key, 1, httpLimit.time());
            }
        }
        return true;
        
    }

    
    
    
    
    private String getIpAddress(HttpServletRequest request) {
        String ip = request.getHeader("x-forwarded-for");
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("Proxy-Client-IP");
        }
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("WL-Proxy-Client-IP");
        }
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("HTTP_CLIENT_IP");
        }
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("HTTP_X_FORWARDED_FOR");
        }
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getRemoteAddr();
        }
        return ip;
    }
}
